"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.errorReporter = void 0;
const pdjs_1 = require("@pagerduty/pdjs");
const web_api_1 = require("@slack/web-api");
const logger_1 = require("./logger");
const errorReporter = (pdApiKey, slackToken, channel, log = logger_1.consoleLogger) => {
    const pd = (0, pdjs_1.api)({ token: pdApiKey });
    const client = new web_api_1.WebClient(slackToken);
    const sendSlackMessage = (message, channel) => __awaiter(void 0, void 0, void 0, function* () {
        return new Promise((resolve, reject) => __awaiter(void 0, void 0, void 0, function* () {
            try {
                const resp = yield client.chat.postMessage({
                    //blocks: message,
                    text: message,
                    channel,
                });
                if (resp.errors) {
                    return reject(new Error(`err sending slack msg: ${resp.errors.join(";")}`));
                }
                if (resp.error) {
                    return reject(new Error(`err sending slack msg: ${resp.error}`));
                }
                log.debug(`sent slack msg ${JSON.stringify(resp.message)}`);
                return resolve(true);
            }
            catch (error) {
                log.error(`exc sending slack msg ${JSON.stringify(error)}`);
                return resolve(true);
            }
        }));
    });
    return {
        report(f) {
            var _a, _b;
            return __awaiter(this, void 0, void 0, function* () {
                for (const t of f.targets) {
                    if (t == 'slack') {
                        yield sendSlackMessage(f.msg, channel);
                    }
                    else {
                        const opt = f.id ? { incident_key: f.id } : {};
                        let res = yield pd.post('/incidents', {
                            headers: {
                                'From': 'alex@renderfoundation.com',
                            },
                            data: {
                                incident: Object.assign(Object.assign({}, opt), { type: "incident", title: (_a = f.title) !== null && _a !== void 0 ? _a : ``, service: {
                                        "id": "PZPJ8EI",
                                        "type": "service_reference"
                                    }, urgency: "high", body: {
                                        "type": "incident_body",
                                        "details": (_b = f.msg) !== null && _b !== void 0 ? _b : ``
                                    } })
                            }
                        });
                        log.debug(`pd resp ${JSON.stringify(res.data)}`);
                        if (res.data.error) {
                            log.error(`pd err ${JSON.stringify(res.data.error)}`);
                        }
                    }
                }
            });
        }
    };
};
exports.errorReporter = errorReporter;
//# sourceMappingURL=errorReporter.js.map