"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.executeSqV3Transaction = exports.executeSqV4Transaction = void 0;
const sdk_1 = require("@sqds/sdk");
const bn_js_1 = __importDefault(require("bn.js"));
const web3_js_1 = require("@solana/web3.js");
const spl_utils_1 = require("@render-foundation/spl-utils");
const logger_1 = require("./logger");
const multisig_1 = require("@render-foundation/multisig");
//import {hel} from "@render-foundation/spl-utils/lib/types"
const multisig = __importStar(require("@render-foundation/multisig"));
const multisig_2 = require("@render-foundation/multisig");
const executeSqV4Transaction = (connection, multisigPda, approvers, instrs, log = logger_1.consoleLogger) => __awaiter(void 0, void 0, void 0, function* () {
    let multisigAccount = yield multisig_1.generated.Multisig.fromAccountAddress(connection, multisigPda);
    let vaultPda = multisig.getVaultPda({
        multisigPda,
        index: 0,
        programId: multisig_2.PROGRAM_ID,
    })[0];
    const transactionIndex = multisig.utils.toBigInt(multisigAccount.transactionIndex) + BigInt(1);
    const txPDA = multisig.getTransactionPda({
        multisigPda,
        index: transactionIndex
    })[0];
    const txMessage = new web3_js_1.TransactionMessage({
        payerKey: vaultPda,
        recentBlockhash: (yield connection.getLatestBlockhash()).blockhash,
        instructions: instrs
    });
    const createIx = multisig.instructions.vaultTransactionCreate({
        multisigPda,
        transactionIndex,
        creator: approvers[0].publicKey,
        vaultIndex: 0,
        ephemeralSigners: 0,
        transactionMessage: txMessage,
        memo: `execute ${txPDA}`,
        programId: multisig_2.PROGRAM_ID,
    });
    // Create Proposal account.
    const createPropIx = multisig.instructions.proposalCreate({
        multisigPda,
        transactionIndex,
        creator: approvers[0].publicKey,
        programId: multisig.PROGRAM_ID,
    });
    //await connection.confirmTransaction(signature);
    const createTx = new web3_js_1.Transaction().add(createIx, createPropIx);
    for (const approver of approvers) {
        createTx.add(multisig.instructions.proposalApprove({
            multisigPda,
            transactionIndex,
            member: approver.publicKey,
            memo: `${approver.publicKey} approve`,
            programId: multisig.PROGRAM_ID,
        }));
    }
    createTx.recentBlockhash = (yield connection.getLatestBlockhash()).blockhash;
    createTx.feePayer = approvers[0].publicKey;
    createTx.sign(...approvers);
    yield (0, spl_utils_1.sendAndConfirmWithRetry)(connection, createTx.serialize(), {
        skipPreflight: true,
        maxRetries: 10,
    }, 'finalized');
    log.info(`created tx ${txPDA}`);
    log.info(`executing tx ${txPDA}`);
    const execTx = yield multisig.transactions.vaultTransactionExecute({
        connection: connection,
        feePayer: approvers[0].publicKey,
        blockhash: (yield connection.getLatestBlockhash()).blockhash,
        multisigPda,
        transactionIndex,
        member: approvers[0].publicKey,
        programId: multisig_2.PROGRAM_ID
    });
    execTx.sign([approvers[0]]);
    const { txid } = yield (0, spl_utils_1.sendAndConfirmWithRetry)(connection, Buffer.from(execTx.serialize()), {
        skipPreflight: true,
        maxRetries: 10,
    }, 'finalized');
    log.info(`executed tx ${txPDA}`);
    return { txSig: txid, txPDA };
});
exports.executeSqV4Transaction = executeSqV4Transaction;
const executeSqV3Transaction = (connection, squadsMsKey, clients, instrs, log = logger_1.consoleLogger) => __awaiter(void 0, void 0, void 0, function* () {
    const msAuthorityK = (0, sdk_1.getAuthorityPDA)(squadsMsKey, new bn_js_1.default(1), sdk_1.DEFAULT_MULTISIG_PROGRAM_ID)[0];
    let txPDA;
    const primClient = clients[0];
    const txIdx = yield primClient.getNextTransactionIndex(squadsMsKey);
    txPDA = (0, sdk_1.getTxPDA)(squadsMsKey, new bn_js_1.default(txIdx), sdk_1.DEFAULT_MULTISIG_PROGRAM_ID)[0];
    let createTx = new web3_js_1.Transaction()
        .add(yield primClient.buildCreateTransaction(squadsMsKey, 1, txIdx));
    let instrIdx = 1;
    for (const i of instrs) {
        createTx.add(yield primClient.buildAddInstruction(squadsMsKey, txPDA, i, instrIdx));
        instrIdx += 1;
    }
    createTx.add(yield primClient.buildActivateTransaction(squadsMsKey, txPDA));
    for (const c of clients) {
        createTx.add(yield c.buildApproveTransaction(squadsMsKey, txPDA));
    }
    createTx.recentBlockhash = (yield connection.getLatestBlockhash()).blockhash;
    createTx.feePayer = primClient.wallet.publicKey;
    for (const c of clients) {
        createTx = yield c.wallet.signTransaction(createTx);
    }
    //createTx.sign(sharedMsKp, authorityKp)
    yield (0, spl_utils_1.sendAndConfirmWithRetry)(connection, createTx.serialize(), {
        skipPreflight: true,
        maxRetries: 10,
    }, 'finalized');
    log.info(`created tx ${txPDA}`);
    log.info(`executing tx ${txPDA}`);
    let execTx = new web3_js_1.Transaction().add(yield primClient.buildExecuteTransaction(txPDA, primClient.wallet.publicKey));
    execTx.recentBlockhash = (yield connection.getLatestBlockhash()).blockhash;
    execTx.feePayer = primClient.wallet.publicKey;
    execTx = yield primClient.wallet.signTransaction(execTx);
    //execTx.sign(args.solFeePayerKp)
    const { txid } = yield (0, spl_utils_1.sendAndConfirmWithRetry)(connection, execTx.serialize(), {
        skipPreflight: true,
        maxRetries: 10,
    }, 'confirmed');
    log.info(`executed ${txid}`);
    return { txSig: txid, txPDA };
});
exports.executeSqV3Transaction = executeSqV3Transaction;
//# sourceMappingURL=squads.js.map