import { api } from "@pagerduty/pdjs";
import { WebClient } from "@slack/web-api";
import { consoleLogger } from "./logger";
export const errorReporter = (pdApiKey, slackToken, channel, log = consoleLogger) => {
    const pd = api({ token: pdApiKey });
    const client = new WebClient(slackToken);
    const sendSlackMessage = async (message, channel) => {
        return new Promise(async (resolve, reject) => {
            try {
                const resp = await client.chat.postMessage({
                    //blocks: message,
                    text: message,
                    channel,
                });
                if (resp.errors) {
                    return reject(new Error(`err sending slack msg: ${resp.errors.join(";")}`));
                }
                if (resp.error) {
                    return reject(new Error(`err sending slack msg: ${resp.error}`));
                }
                log.debug(`sent slack msg ${JSON.stringify(resp.message)}`);
                return resolve(true);
            }
            catch (error) {
                log.error(`exc sending slack msg ${JSON.stringify(error)}`);
                return resolve(true);
            }
        });
    };
    return {
        async report(f) {
            for (const t of f.targets) {
                if (t == 'slack') {
                    await sendSlackMessage(f.msg, channel);
                }
                else {
                    const opt = f.id ? { incident_key: f.id } : {};
                    let res = await pd.post('/incidents', {
                        headers: {
                            'From': 'alex@renderfoundation.com',
                        },
                        data: {
                            incident: {
                                ...opt,
                                type: "incident",
                                title: f.title ?? ``,
                                service: {
                                    "id": "PZPJ8EI",
                                    "type": "service_reference"
                                },
                                urgency: "high",
                                body: {
                                    "type": "incident_body",
                                    "details": f.msg ?? ``
                                },
                            }
                        }
                    });
                    log.debug(`pd resp ${JSON.stringify(res.data)}`);
                    if (res.data.error) {
                        log.error(`pd err ${JSON.stringify(res.data.error)}`);
                    }
                }
            }
        }
    };
};
//# sourceMappingURL=errorReporter.js.map