import * as anchor from "@coral-xyz/anchor";
import { CONTRACTS } from "@certusone/wormhole-sdk";
import { consoleLogger } from "./logger";
export const fetchEthInfo = async (seq, wormEnv = 'MAINNET', ethChainId = 2, retries = 3, log = consoleLogger) => {
    const hexEmitter = Buffer.alloc(32)
        .fill(Uint8Array.from(anchor.utils.bytes.hex.decode(CONTRACTS[wormEnv].ethereum.token_bridge)), 12)
        .toString('hex');
    const url = `https://api${wormEnv.toLowerCase() == 'mainnet' ? '' : `.${wormEnv.toString().toLowerCase()}`}.wormholescan.io/api/v1/global-tx/${ethChainId}/${hexEmitter}/${seq}`;
    const recur = async (currRetry = 0) => {
        if (currRetry == retries) {
            return Promise.reject(new Error(`wormapi exceeded max retries ${retries} fetching ${url}`));
        }
        try {
            const res = await fetch(url).then(async (res) => {
                if (res.status != 200) {
                    throw new Error(`status code ${res.status} fetching ${url}. body: ${JSON.stringify(await res.json())}`);
                }
                return (await res.json());
            }, err => {
                throw new Error(`exception fetching wormapi: ${JSON.stringify(err)}`);
            });
            return [seq, res];
        }
        catch (e) {
            // @ts-ignore
            log.error(`wormapi err ${e.message}`);
            return recur(currRetry + 1);
        }
    };
    return (await recur());
};
//# sourceMappingURL=wormhole.js.map