import type { ColumnType } from "kysely";
export type Generated<T> = T extends ColumnType<infer S, infer I, infer U> ? ColumnType<S, I | undefined, U> : ColumnType<T, T | undefined, T>;
export type Int8 = ColumnType<string, bigint | number | string, bigint | number | string>;
export type Timestamp = ColumnType<Date, Date | string, Date | string>;
export type TransferType = "node_operator_rewards" | "open_compute_rewards" | "upgrade_rewards";
export type TxStatus = "created" | "executed" | "ready";
export interface Burn {
    burned: Int8;
    executed_at: Timestamp | null;
    id: Generated<Int8>;
    marked_burned_at: Timestamp | null;
    priced_at: Timestamp | null;
    quote_amt: Int8 | null;
    sol_tx: string;
    usdc_spent: Int8 | null;
}
export interface Job {
    burn_id: Generated<Int8>;
    completed_at: Timestamp | null;
    id: Generated<Int8>;
    render_amt: Int8;
}
export interface JobId {
    epoch_id: Generated<Int8>;
    id: Int8;
}
export interface Lock {
    acquired_at: Timestamp | null;
    cursor: string | null;
    description: string | null;
    id: Generated<Int8>;
    released_at: Timestamp | null;
}
export interface NodeOperatorEpoch {
    burn_tx: string | null;
    burned: Generated<Int8>;
    description: string | null;
    end_time: Timestamp | null;
    id: Generated<Int8>;
    saved_at: Timestamp | null;
    start_time: Timestamp | null;
    total_rndr: Generated<Int8>;
}
export interface NodeOperatorRewardsTransfer {
    end_time: Timestamp | null;
    epoch_id: Generated<Int8>;
    start_time: Timestamp | null;
    transfer_id: Generated<Int8>;
}
export interface OpenComputeRewardsTransfer {
    device_id: string | null;
    epoch_id: Generated<Int8>;
    transfer_id: Generated<Int8>;
}
export interface SolTx {
    executed_at: Timestamp | null;
    id: Generated<Int8>;
    sig: string;
}
export interface SolTxTransfer {
    amount: Int8 | null;
    sol_tx_id: Generated<Int8>;
    transfer_id: Generated<Int8>;
}
export interface SquadsTx {
    id: Generated<Int8>;
    sq_tx: string;
    tx_status: TxStatus;
}
export interface Transfer {
    amount_due: Int8;
    amount_payed: Int8;
    created_at: Generated<Timestamp>;
    id: Generated<Int8>;
    payed_at: Timestamp | null;
    sol_key: string | null;
    sol_tx: string | null;
    sq_tx_id: Int8 | null;
    transfer_type: TransferType;
}
export interface UpgradeRewardsTransfer {
    epoch_id: Generated<Int8>;
    epoch_start_time: Timestamp | null;
    points: Int8 | null;
    transfer_id: Generated<Int8>;
}
export interface DB {
    burn: Burn;
    job: Job;
    job_id: JobId;
    lock: Lock;
    node_operator_epoch: NodeOperatorEpoch;
    node_operator_rewards_transfer: NodeOperatorRewardsTransfer;
    open_compute_rewards_transfer: OpenComputeRewardsTransfer;
    sol_tx: SolTx;
    sol_tx_transfer: SolTxTransfer;
    squads_tx: SquadsTx;
    transfer: Transfer;
    upgrade_rewards_transfer: UpgradeRewardsTransfer;
}
//# sourceMappingURL=dbTypes.d.ts.map